class Robot:  # お手伝いロボクラス
    # 初期化メソッド
    def __init__(self, name, battery):
        # プロパティ（_（アンダーバー）で紳士的なカプセル化）
        self._job = "お手伝い"
        self._name = name
        self._battery = battery

    # ゲッターメソッド（読み取り専用で、カプセル化されているプロパティの値をクラス外部から確認できる）
    @property
    def job(self):
        return self._job

    @property
    def name(self):
        return self._name

    @property
    def battery(self):
        return self._battery

    # セッターメソッド（カプセル化されているプロパティの値をクラス外部から変更できる）
    @battery.setter
    def battery(self, value):
        if 0 <= value <= 100:
            self._battery = value
        else:
            print("注意：batteryの値は、0~100%の間にして下さい。")

    # メソッド
    def show_info(self):  # ロボの情報を報告するメソッド
        # f文字列は、format()メソッドよりも短い記法で文字列内に数値や式、変数を埋め込むことができる。また、出力する数値や式、変数ごとに書式設定することもできる。
        print(
            f"私は「{self._name}」です。仕事は{self._job}で、充電残量は{self._battery:.1f}%です。"
        )

    def do_work(self):  # ロボに仕事をさせるメソッド
        self._battery -= 10
        print(
            f"{self._name}は、お手伝いをしました。充電残量は{self._battery:.1f}%です。"
        )


class CookingRobot(Robot):  # お料理ロボクラス（お手伝いロボクラスを継承）
    def __init__(self, name, battery):
        super().__init__(name, battery)
        self._job = "お料理"

    # ロボに仕事をさせるメソッド（ポリモーフィズム）。親クラスから継承したメソッドを子クラスで上書き。
    def do_work(self):
        self._battery -= 20
        print(f"{self._name}は、昼食を作りました。充電残量は{self._battery:.1f}%です。")


class CleaningRobot(Robot):  # お掃除ロボクラス（お手伝いロボクラスを継承）
    def __init__(self, name, battery):
        super().__init__(name, battery)
        self._job = "お掃除"

    # ロボに仕事をさせるメソッド（ポリモーフィズム）。親クラスから継承したメソッドを子クラスで上書き。
    def do_work(self):
        self._battery -= 30
        print(
            f"{self._name}は、庭の掃除をしました。充電残量は{self._battery:.1f}%です。"
        )


"""
robot1 = Robot("ココア", 100)  # Robotクラスからrobot1インスタンスを作成
robot2 = Robot("ラテ", 80.00)
robot1.battery = 10000
# robot1._battery = 10000
robot1.battery = 90
robot1.show_info()  # robot1にメソッドを命令
robot2.show_info()
print("仕事を命令！")
robot1.do_work()
robot2.do_work()
"""
robots = []
robots.append(Robot("ココア", 100))
robots.append(Robot("ラテ", 80.00))
robots.append(CookingRobot("シェフィ", 100))
robots.append(CleaningRobot("スウィーピー", 100))
for robot in robots:
    robot.show_info()
print("\n仕事を命令！")
for robot in robots:
    robot.do_work()
