# ゲームをコントロールするプログラム
import enemy
import player
import pygame as pg


class GameManager:  # ゲーム管理用のクラス
    # 初期化メソッド
    def __init__(self, screen):
        # プロパティ
        # 主人公（playerインスタンス）を作成
        self._player = player.Player()
        # 敵（enemyインスタンス）を格納するための空のリストを作成
        self._enemies = []
        # 爆発エフェクト（BombEffectインスタンス）を格納するための空のリストを作成。一度に複数爆発することもあるため、リストに格納する。
        self._effects = []
        # プロパティをGameManagerインスタンスのreset()メソッドで初期化する
        self.reset(screen)

    @property  # _is_playingプロパティに対するゲッターメソッド
    def is_playing(self):
        return self._is_playing

    @property  # _is_clearedプロパティに対するゲッターメソッド
    def is_cleared(self):
        return self._is_cleared

    # メソッド
    def reset(self, screen):  # ゲームのリセット
        # 今はゲーム中か、敵が落下エリアに落ちてゲームが停止中かどうかを知らせるためのフラグとなるプロパティ。通常はTrue、停止中はFalseにする。
        self._is_playing = True
        # ゲームクリアしたかどうかのフラグとなるプロパティ
        self._is_cleared = False
        self._player.reset()  # Playerインスタンスのreset()メソッド
        self._enemies.clear()  # リストのclear()メソッドで敵リストを空にする
        """
        for i in range(8):  # 敵を作成し、リストに追加する✕８
            self._enemies.append(enemy.Enemy(screen))
        """
        for i in range(2):  # 通常の敵を２体作成
            self._enemies.append(enemy.Enemy(screen))
        for i in range(1):  # 炎の敵を１体作成
            self._enemies.append(enemy.FlameEnemy(screen))
        for i in range(1):  # 氷の敵を１体作成
            self._enemies.append(enemy.IceEnemy(screen))

    def update(self, screen):  # 更新処理
        for e in self._effects:
            e.update()  # 爆発エフェクトリストの爆発エフェクトを更新
        self._player.update(screen)  # 主人公の更新
        key = pg.key.get_pressed()  # 押されているキーを調べて変数keyに代入

        # 主人公と敵が衝突するとHPを少し減らし、HPがゼロになったら敵を倒し、全ての敵を倒したらゲームクリア状態になる処理を行う関数
        def gameClear(e):
            e.hp -= 50  # 敵のHPを少し減らす
            if e.hp <= 0:  # 敵のHPが０になったら
                # 引数e.rectは、敵（Enemyインスタンス）の配置位置（Rectオブジェクト）
                bomb = enemy.BombEffect(e.rect, self._effects)
                self._effects.append(bomb)
                self._enemies.remove(e)
                if len(self._enemies) == 0:  # 敵の残りが０になったら
                    self._is_playing = False  # ゲームを停止する
                    self._is_cleared = True  # ゲームクリア
                return

        for e in self._enemies:
            e.update(screen)  # 敵の更新
            if e.rect.y >= screen.get_height() - 55:
                self._is_playing = False  # ゲームを停止する
            # 敵と主人公が衝突したら、敵を上に移動し、敵の向きを変更する
            # pygame.Rect.colliderect(Rectオブジェクト)メソッドは、２つのRectオブジェクトが重なっているかどうか調べる。２つのRectオブジェクトの一部分が重なっている場合にはTrueが返される。(重なっているのが上辺と下辺、もしくは、右辺と左辺の場合は重なっているとは見なされない)
            if e.rect.colliderect(self._player.rect):
                if key[pg.K_RIGHT]:  # 右キー押下時に敵と衝突した場合
                    e.rect.x = self._player.rect.x + 100
                    e.vy = -abs(e.vy)
                    gameClear(e)
                elif key[pg.K_LEFT]:  # 左キー押下時に敵と衝突した場合
                    e.rect.x = self._player.rect.x - 100
                    e.vy = -abs(e.vy)
                    gameClear(e)
                elif key[pg.K_UP]:  # 上キー押下時に敵と衝突した場合
                    e.rect.y = self._player.rect.y - 50
                    e.vy = -abs(e.vy)
                    gameClear(e)
                elif key[pg.K_DOWN]:  # 下キー押下時に敵と衝突した場合
                    e.rect.y = self._player.rect.y + 75
                    gameClear(e)
                else:  # キーを押していない状態で敵と衝突した場合
                    e.rect.y = self._player.rect.y - 50
                    e.vx = -abs(e.vx)
                    e.vy = -abs(e.vy)
                    gameClear(e)

    def draw(self, screen):  # 描画処理
        for e in self._effects:  # 爆発エフェクトの描画
            e.draw(screen)
        self._player.draw(screen)  # 主人公の描画
        for e in self._enemies:
            e.draw(screen)  # 敵の描画
