# 主人公（Player）クラスのプログラムファイル
import pygame as pg


class Player:  # 主人公作成用のクラス
    # 初期化メソッド
    def __init__(self):
        # プロパティ
        self.reset()

    @property  # _rectプロパティに対するゲッターメソッド
    def rect(self):
        return self._rect

    @rect.setter  # _rectプロパティに対するセッターメソッド
    def rect(self, value):
        self._rect = value

    # メソッド
    def reset(self):  # 主人公のリセット
        # プロパティ
        self._images = [
            pg.image.load("images/kaeru1.png"),
            pg.image.load("images/kaeru2.png"),
            pg.image.load("images/kaeru3.png"),
            pg.image.load("images/kaeru4.png"),
        ]  # 主人公のアニメーション画像ファイル
        self._cnt = 0  # 描画回数をカウントする変数
        self._image = self._images[0]  # 主人公の初期状態での画像ファイル
        # 画像ファイルが描画されるRectオブジェクトの位置・サイズを指定
        self._rect = pg.Rect(250, 550, 50, 50)  # 表示位置
        self._speed = 10  # 移動速度

    def update(self, screen):  # 更新処理
        key = pg.key.get_pressed()  # 押されているキーを調べて変数keyに代入
        vx = 0  # X軸方向の移動量
        vy = 0  # Y軸方向の移動量
        if key[pg.K_RIGHT]:  # 右キー
            vx = self._speed
        elif key[pg.K_LEFT]:  # 左キー
            vx = -self._speed
        elif key[pg.K_UP]:  # 上キー
            vy = -self._speed
        elif key[pg.K_DOWN]:  # 下キー
            vy = self._speed
        elif self._rect.x + vx < 0:  # 移動した位置が画面の左端を超えたら
            vx = self._speed
        # 移動した位置が画面の右端を超えたら
        elif self._rect.x + vx > screen.get_width() - 50:
            vx = -50
        elif self._rect.y + vy < 0:  # 移動した位置が画面の上端を超えたら
            vy = self._speed
        # 移動した位置が画面の下端を超えたら
        elif self._rect.y + vy > screen.get_height() - 50:
            vy = -50
        self._rect.x += vx  # 画像の表示位置にX軸方向の移動量を追加
        self._rect.y += vy  # 画像の表示位置にY軸方向の移動量を追加
        self._cnt += 1
        # 描画回数（self._cnt）を５で割って５回に１回のゆっくりな画像の切り替えを行い、その値を４で割った余りで０～３番目の画像を指定する
        self._image = self._images[self._cnt // 5 % 4]

    def draw(self, screen):  # 描画処理
        screen.blit(self._image, self._rect)  # 画像ファイルを描画
