# 主人公（Player）クラスのプログラムファイル
import pygame as pg


class PlayerState:  # 主人公の状態の基本形クラス
    def __init__(self, player):
        # プロパティ
        self._player = player
        self._image = None

    def update(self):  # 更新処理
        pass

    @property
    def image(self):
        return self._image


class IdleState(PlayerState):  # 主人公の待機状態のクラス
    def __init__(self, player):
        super().__init__(player)
        # プロパティ
        # 主人公の待機状態画像ファイル
        self._image = pg.image.load("images/kaeru1.png")

    # メソッド
    def update(self):  # 更新処理
        key = pg.key.get_pressed()  # 押されているキーを調べて変数keyに代入
        if key[pg.K_RIGHT] or key[pg.K_LEFT] or key[pg.K_UP] or key[pg.K_DOWN]:
            return MovingState(self._player)  # 主人公の移動状態
        else:
            return self


class MovingState(PlayerState):  # 主人公の移動状態のクラス
    def __init__(self, player):
        super().__init__(player)
        # プロパティ
        self._images = [
            pg.image.load("images/kaeru1.png"),
            pg.image.load("images/kaeru2.png"),
            pg.image.load("images/kaeru3.png"),
            pg.image.load("images/kaeru4.png"),
        ]  # 主人公の移動状態アニメーション画像ファイル
        self._cnt = 0  # 描画回数をカウントする変数
        self._image = self._images[0]  # 主人公の移動状態での初期画像

    def update(self):  # 更新処理
        self._cnt += 1
        # 描画回数（self._cnt）を５で割って５回に１回のゆっくりな画像の切り替えを行い、その値を４で割った余りで０～３番目の画像を指定する
        self._image = self._images[self._cnt // 5 % 4]
        key = pg.key.get_pressed()  # 押されているキーを調べて変数keyに代入
        if not (key[pg.K_RIGHT] or key[pg.K_LEFT] or key[pg.K_UP] or key[pg.K_DOWN]):
            return IdleState(self._player)  # 主人公の待機状態
        else:
            return self


class DamageState(PlayerState):  # 主人公のダメージ状態のクラス
    def __init__(self, player):
        super().__init__(player)
        # プロパティ
        self._images = [
            pg.image.load("images/kaeru5.png"),
            pg.image.load("images/kaeru6.png"),
        ]  # 主人公のダメージ状態アニメーション画像ファイル
        self._cnt = 0  # 描画回数をカウントする変数
        self._image = self._images[0]  # # 主人公のダメージ状態での初期画像
        self._timeout = 20  # ダメージ状態から待機状態に戻るまでのカウント

    def update(self):  # 更新処理
        self._cnt += 1
        self._image = self._images[self._cnt // 5 % 2]
        # タイムアウトチェック（ダメージ状態でしばらくしたら待機状態に戻る処理）
        self._timeout -= 1
        if self._timeout < 0:
            return IdleState(self._player)  # 主人公の待機状態
        else:
            return self


class Player:  # 主人公作成用のクラス
    # 初期化メソッド
    def __init__(self):
        # プロパティ
        self.reset()

    @property  # _rectプロパティに対するゲッターメソッド
    def rect(self):
        return self._rect

    @rect.setter  # _rectプロパティに対するセッターメソッド
    def rect(self, value):
        self._rect = value

    # メソッド
    def reset(self):  # 主人公のリセット
        # プロパティ
        # 主人公の現在の状態クラスを入れるためのプロパティ
        self._state = IdleState(self)  # 初期状態として待機状態を入れる
        # 画像ファイルが描画されるRectオブジェクトの位置・サイズを指定
        self._rect = pg.Rect(250, 550, 50, 50)  # 表示位置
        self._speed = 10  # 移動速度
        self._maxhp = 150  # 最大HP
        self._hp = 150  # 現在のHP

    @property  # _maxhpプロパティに対するゲッターメソッド
    def maxhp(self):
        return self._maxhp

    @property  # _hpプロパティに対するゲッターメソッド
    def hp(self):
        return self._hp

    @hp.setter  # _hpプロパティに対するセッターメソッド
    def hp(self, value):
        self._hp = value

    def update(self, screen):  # 更新処理
        # 待機状態と移動状態間での切り替えを行う
        self._state = self._state.update()
        key = pg.key.get_pressed()  # 押されているキーを調べて変数keyに代入
        vx = 0  # X軸方向の移動量
        vy = 0  # Y軸方向の移動量
        if key[pg.K_RIGHT]:  # 右キー
            vx = self._speed
        elif key[pg.K_LEFT]:  # 左キー
            vx = -self._speed
        elif key[pg.K_UP]:  # 上キー
            vy = -self._speed
        elif key[pg.K_DOWN]:  # 下キー
            vy = self._speed
        elif self._rect.x + vx < 0:  # 移動した位置が画面の左端を超えたら
            vx = self._speed
        # 移動した位置が画面の右端を超えたら
        elif self._rect.x + vx > screen.get_width() - 50:
            vx = -50
        elif self._rect.y + vy < 0:  # 移動した位置が画面の上端を超えたら
            vy = self._speed
        # 移動した位置が画面の下端を超えたら
        elif self._rect.y + vy > screen.get_height() - 50:
            vy = -50
        self._rect.x += vx  # 画像の表示位置にX軸方向の移動量を追加
        self._rect.y += vy  # 画像の表示位置にY軸方向の移動量を追加

    def draw(self, screen):  # 描画処理
        # 引数self._state.imageは、主人公の各状態の_imageプロパティ
        screen.blit(self._state.image, self._rect)  # 画像ファイルを描画
        # HPバー
        # 最大HPバーが描画されるRectオブジェクトの位置・サイズを指定
        maxhp_rect = pg.Rect(self._rect.x, self._rect.y - 20, 4, 20)
        hp_height = (self._hp / self._maxhp) * 20  # 現在のHPバーの高さ
        hp_rect = pg.Rect(
            self._rect.x, self._rect.y - hp_height, 4, hp_height
        )  # 現在のHPバーが描画されるRectオブジェクトの位置・サイズを指定
        pg.draw.rect(screen, pg.Color("RED"), maxhp_rect)  # 最大HPバー
        pg.draw.rect(screen, pg.Color("GREEN"), hp_rect)  # 現在のHPバー

    def damage(self):  # ダメージ化
        # reset()メソッドが保持している主人公の現在の状態クラスを入れるためのプロパティ
        self._state = DamageState(self)  # ダメージ状態のクラスに切り替える
