# 音楽の再生を管理するクラス
import random

import pygame as pg


# シングルトンパターン
class SoundManager:  # サウンドマネージャー
    _instance = None  # クラス変数

    @classmethod  # クラスメソッド（引数「cls」には自身のクラスが渡される）
    def get_instance(cls):  # １つだけのインスタンスを取得
        # インスタンス未生成の場合（_instanceの中身がない場合）
        if cls._instance is None:
            cls._instance = cls()  # クラス名()でインスタンスを作成
        return cls._instance

    # 初期化メソッド
    def __init__(self):
        # mixerモジュールは、音声を読み込んで再生（再生前に全てのファイル読み込む必要がある）するためのモジュール。musicモジュールは、mixerモジュールと密接につながっており、サウンドミキサーでの音楽再生（ストリーミング再生：再生前に全てのファイル読み込む必要はない）を操作できる。
        # pygame.mixer.music.load()メソッドは、再生する音楽ファイルや音楽オブジェクトを読み込んで、再生の準備をする。
        pg.mixer.music.load("sounds/bgm.wav")
        # プロパティ
        # pygame.mixer.Sound()メソッドは、ファイルなどからSoundオブジェクトを新規に作成する。作成されたSoundオブジェクトは、再生される音声のサンプリングデータを表す。
        self._over = pg.mixer.Sound("sounds/over.wav")
        self._clear = pg.mixer.Sound("sounds/clear.wav")
        self._clap1 = pg.mixer.Sound("sounds/clap1.wav")
        self._clap2 = pg.mixer.Sound("sounds/clap2.wav")
        self._clap3 = pg.mixer.Sound("sounds/clap3.wav")
        self._blast = pg.mixer.Sound("sounds/blast.wav")
        self._bomb = pg.mixer.Sound("sounds/bomb.wav")

    # メソッド
    def bgmstart(self):  # BGM再生
        # pygame.mixer.music.play(loops引数,start引数)メソッドは、読み込んだ音楽をストリーミング再生する。loops引数は、初回の再生後に繰り返す再生回数を指定し、トータルの再生回数は指定した回数プラス１回再生される。start引数は、音楽の再生開始位置を指定できる。
        pg.mixer.music.play(-1)  # loops引数に「-1」を指定するとループ再生

    def bgmstop(self):  # BGM停止
        # pygame.mixer.music.stop()メソッドは、音楽の再生を終了する。
        pg.mixer.music.stop()

    def playover(self):  # ゲームオーバー音
        # pygame.mixer.Sound.play(loops引数,maxtime引数,fade_ms引数)メソッドは、音声の再生を開始する。loops引数については、music.play()メソッドと同じ内容で、maxtime引数は、音声の再生時間を指定できる。fade_ms引数は、音量を０の状態で再生を開始して指定した時間の間に徐々に音量をっ既定値まで上げていくことができる。
        self._over.play()

    def playclear(self):  # ゲームクリア音
        self._clear.play()

    def playattack(self):  # 主人公の攻撃音
        r = random.randint(0, 3)
        if r == 0:
            self._clap1.play()
        elif r == 1:
            self._clap2.play()
        else:
            self._clap3.play()

    def playblast(self):  # 敵を破壊した時の音
        self._blast.play()

    def playbomb(self):  # 主人公がダメージを受けた時の音
        self._bomb.play()
